// Look for "ADD HERE" in this file

// This example reads the ambient light using the Maxim MAX44000PMB1 Pmod

/* MAX44000PMB1 plugged into FMC Carrier JA, using PL AXI_IIC
 *
 */

#include <stdio.h>
#include "platform.h"
#include "xparameters.h"
#include "sleep.h"
/* ADD HERE
 * What other header file do you need?
 */
#include "xiic_l.h"

#define MAX44000_IIC_ADDRESS            0x4A   //!< The actual address is 8'b1001 010x (0x94)
#define MAX44000_MAIN_CONFIG_REG        0x01   // Main configuration register
#define MAX44000_RECV_CONFIG_REG        0x02   // Receive configuration register
#define MAX44000_LED_DRIVE_CURRENT_REG  0x03   // LED Drive Current register
#define MAX44000_LUX_HIGH_BYTE_REG      0x04   // Lux High-Byte register (overflow bit + bits 13:8 of lux reading)
#define MAX44000_LUX_LOW_BYTE_REG       0x05   // Lux Low-Byte register (bits 7:0 of lux reading)

int MAX44000_write(u32 ZynqIicAddress, u8 register_offset, u8 write_value);
int MAX44000_read(u32 ZynqIicAddress, u8 register_offset, u8 *read_value);

int main()
{
	u8 LuxHighByte=0;
	u8 LuxLowByte=0;
	unsigned int LuxReading=0;
	u8 loop;

    init_platform();
	/* ADD HERE
	 *  Add code and variables, if necessary, to set up the IIC controller.
	 */

    // None required for the low-level AXI IIC

    printf("Hello MicroZed\n\r");

    // Set to Ambient Light Sensor mode
    MAX44000_write(XPAR_IIC_0_BASEADDR, MAX44000_MAIN_CONFIG_REG, 0x24);

    // Set the Ambient Light Sensor programmable gain
    // Value = 0x00 sets up 14-bit resolution, gain = 0.03125 LUX/lsb
    MAX44000_write(XPAR_IIC_0_BASEADDR, MAX44000_RECV_CONFIG_REG, 0x00);

    // Now take 30 readings, 1 second apart
    // Experiment covering and uncovering the device to see if it really
    //   is detecting the changes in light. Use a flashlight to
    //   force the device to overflow. What happens? How would you fix it?
    for(loop=0; loop<30; loop++)
    {
       sleep(1); // sleep 1s
       MAX44000_read(XPAR_IIC_0_BASEADDR, MAX44000_LUX_HIGH_BYTE_REG, &LuxHighByte);
       MAX44000_read(XPAR_IIC_0_BASEADDR, MAX44000_LUX_LOW_BYTE_REG, &LuxLowByte);
       if((LuxHighByte & 0x40)==0x40)
          printf("Overflow occurred\r\n");
       LuxReading = (LuxHighByte & 0x3F);
       LuxReading = LuxReading << 8;
       LuxReading += LuxLowByte;
       LuxReading /= 32; // Undo the previous gain entered
       printf("Lux Value = %d\r\n",LuxReading);
    }

    printf("Testing Complete\n\r");

    return 0;
}

int MAX44000_write(u32 ZynqIicAddress, u8 register_offset, u8 write_value)
/*
* \brief       Use the Zynq IIC Controller to write a value to a
*              MAX44000 register at a given offset
*
* \param[in]   ZynqIicAddress    - address of the I2C Controller
* \param[in]   register_offset   - offset of register inside the MAX44000
* \param[in]   write_value       - value to be written to MAX44000 register
*
* \return      XST_SUCCESS if operation succeeded
*/
{
	int Status = XST_SUCCESS;
	u8 TxBuffer[128]; // Only need this to be size 2, but making larger for future use
	int ByteCount;

	TxBuffer[0] = register_offset;  // Offset of register to write
	TxBuffer[1] = write_value;  // value to write there
	/* ADD HERE
	 *  Add code to send the 2 bytes contained in TxBuffer over I2C to
	 *  the device responding to I2C address MAX44000_IIC_ADDRESS.
	 *  The first byte of TxBuffer contains the offset to the register
	 *  inside the MAX44000. The second byte contains the data to be
	 *  written to that register. If the transfer fails, then
	 *  set Status to XST_FAILURE
	 */
	ByteCount = XIic_Send(ZynqIicAddress, MAX44000_IIC_ADDRESS,
						TxBuffer, 2, XIIC_STOP);
	if(ByteCount!=2)
		Status=XST_FAILURE;

	return(Status);
}

int MAX44000_read(u32 ZynqIicAddress, u8 register_offset, u8 *read_value)
/*
* \brief       Use the Zynq IIC Controller to read a value from a
*              MAX44000 register at a given offset
*
* \param[in]   ZynqIicAddress    - address of the I2C Controller
* \param[in]   register_offset   - offset of register inside the MAX44000
* \param[in]   *read_value       - pointer to data read from MAX44000 register
*
* \return      XST_SUCCESS if operation succeeded
*/
{
	int Status = XST_SUCCESS;
	u8 TxBuffer[128]; // Only need this to be size 1, but making larger for future use
	u8 RxBuffer[128]; // Only need this to be size 1, but making larger for future use
	int ByteCount;

	TxBuffer[0] = register_offset;
	/* ADD HERE
	 *  Add code to send 1 byte contained in TxBuffer over I2C to
	 *  the device responding to I2C address MAX44000_IIC_ADDRESS.
	 *  TxBuffer contains the offset to the register
	 *  inside the MAX44000. If the transfer fails, then
	 *  set Status to XST_FAILURE
	 */
	ByteCount = XIic_Send(ZynqIicAddress, MAX44000_IIC_ADDRESS,
			TxBuffer, 1, XIIC_STOP);
	if(ByteCount!=1)
		Status=XST_FAILURE;

	/* ADD HERE
	 *  Add code to receive 1 byte into RxBuffer over I2C from
	 *  the device responding to I2C address MAX44000_IIC_ADDRESS.
	 *  Since we previously sent the offset to the register
	 *  inside the MAX44000, the MAX44000 will now return the data
	 *  contained within that register. If the transfer fails, then
	 *  set Status to XST_FAILURE
	 */
	ByteCount = XIic_Recv(ZynqIicAddress, MAX44000_IIC_ADDRESS,
			RxBuffer, 1, XIIC_STOP);
	if(ByteCount!=1)
		Status=XST_FAILURE;

	if(Status==XST_SUCCESS)
		*read_value = RxBuffer[0];

	usleep(100000); // Delay 100 ms, which is 100K us
	return(Status);
}
